/*
 * SPDX-FileCopyrightText: 2013-2020 CSSlayer <wengxt@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 *
 */

#ifndef GUICOMMON_H
#define GUICOMMON_H

#define DOWNLOAD_HOST_BASE "download.pinyin.sogou.com"
#define HOST_BASE "pinyin.sogou.com"
#define URL_BASE "http://" HOST_BASE "/dict/"

#endif // GUICOMMON_H
